@@ -17,6 +17,7 @@ import datetime
 
 import copy
 import iso8601
+from oslo_utils import strutils
 from oslo_utils import timeutils
 import six
 
@@ -299,6 +300,12 @@ class Boolean(FieldType):
         return bool(value)
 
 
+class FlexibleBoolean(Boolean):
+    @staticmethod
+    def coerce(obj, attr, value):
+        return strutils.bool_from_string(value)
+
+
 class DateTime(FieldType):
     def __init__(self, tzinfo_aware=True, *args, **kwargs):
         self.tzinfo_aware = tzinfo_aware
@@ -509,10 +516,25 @@ class StringField(AutoTypedField):
     AUTO_TYPE = String()
 
 
-class EnumField(AutoTypedField):
-    def __init__(self, valid_values, **kwargs):
-        self.AUTO_TYPE = Enum(valid_values)
-        super(EnumField, self).__init__(**kwargs)
+class BaseEnumField(AutoTypedField):
+    '''Base class for all enum field types
+
+    This class should not be directly instantiated. Instead
+    subclass it and set AUTO_TYPE to be a SomeEnum()
+    where SomeEnum is a subclass of Enum.
+    '''
+
+    def __init__(self, **kwargs):
+        if self.AUTO_TYPE is None:
+            raise exception.EnumFieldUnset(
+                fieldname=self.__class__.__name__)
+
+        if not isinstance(self.AUTO_TYPE, Enum):
+            raise exception.EnumFieldInvalid(
+                typename=self.AUTO_TYPE.__class__.__name,
+                fieldname=self.__class__.__name__)
+
+        super(BaseEnumField, self).__init__(**kwargs)
 
     def __repr__(self):
         valid_values = self._type._valid_values
@@ -526,6 +548,21 @@ class EnumField(AutoTypedField):
                                      for k, v in sorted(args.items())]))
 
 
+class EnumField(BaseEnumField):
+    '''Anonymous enum field type
+
+    This class allows for anonymous enum types to be
+    declared, simply by passing in a list of valid values
+    to its constructor. It is generally preferrable though,
+    to create an explicit named enum type by sub-classing
+    the BaseEnumField type directly.
+    '''
+
+    def __init__(self, valid_values, **kwargs):
+        self.AUTO_TYPE = Enum(valid_values=valid_values)
+        super(EnumField, self).__init__(**kwargs)
+
+
 class UUIDField(AutoTypedField):
     AUTO_TYPE = UUID()
 
@@ -538,10 +575,21 @@ class FloatField(AutoTypedField):
     AUTO_TYPE = Float()
 
 
+# This is a strict interpretation of boolean
+# values using Python's semantics for truth/falsehood
 class BooleanField(AutoTypedField):
     AUTO_TYPE = Boolean()
 
 
+# This is a flexible interpretation of boolean
+# values using common user friendly semantics for
+# truth/falsehood. ie strings like 'yes', 'no',
+# 'on', 'off', 't', 'f' get mapped to values you
+# would expect.
+class FlexibleBooleanField(AutoTypedField):
+    AUTO_TYPE = FlexibleBoolean()
+
+
 class DateTimeField(AutoTypedField):
     def __init__(self, tzinfo_aware=True, **kwargs):
         self.AUTO_TYPE = DateTime(tzinfo_aware=tzinfo_aware)
